/*
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://x2c.lcm.at/
 * $LastChangedRevision: 2843 $
 * $LastChangedDate:: 2016-02-18 12:04:02 +0100#$
 */
#include "Sin_Data.h"

/*
 * Sine data in 8 bit format.
 * -128..0..127 ~ -Pi..0..Pi/256*255
 */
#ifndef __NO_SIN_TABLE8__
	const int8 Sin_Table8[257]  = {
		   0,    3,    6,    9,   12,   16,   19,   22,
		  25,   28,   31,   34,   37,   40,   43,   46,
		  49,   51,   54,   57,   60,   63,   65,   68,
		  71,   73,   76,   78,   81,   83,   85,   88,
		  90,   92,   94,   96,   98,  100,  102,  104,
		 106,  107,  109,  111,  112,  113,  115,  116,
		 117,  118,  120,  121,  122,  122,  123,  124,
		 125,  125,  126,  126,  126,  127,  127,  127,
		 127,  127,  127,  127,  126,  126,  126,  125,
		 125,  124,  123,  122,  122,  121,  120,  118,
		 117,  116,  115,  113,  112,  111,  109,  107,
		 106,  104,  102,  100,   98,   96,   94,   92,
		  90,   88,   85,   83,   81,   78,   76,   73,
		  71,   68,   65,   63,   60,   57,   54,   51,
		  49,   46,   43,   40,   37,   34,   31,   28,
		  25,   22,   19,   16,   12,    9,    6,    3,
		   0,   -3,   -6,   -9,  -12,  -16,  -19,  -22,
		 -25,  -28,  -31,  -34,  -37,  -40,  -43,  -46,
		 -49,  -51,  -54,  -57,  -60,  -63,  -65,  -68,
		 -71,  -73,  -76,  -78,  -81,  -83,  -85,  -88,
		 -90,  -92,  -94,  -96,  -98, -100, -102, -104,
		-106, -107, -109, -111, -112, -113, -115, -116,
		-117, -118, -120, -121, -122, -122, -123, -124,
		-125, -125, -126, -126, -126, -127, -127, -127,
		-127, -127, -127, -127, -126, -126, -126, -125,
		-125, -124, -123, -122, -122, -121, -120, -118,
		-117, -116, -115, -113, -112, -111, -109, -107,
		-106, -104, -102, -100,  -98,  -96,  -94,  -92,
		 -90,  -88,  -85,  -83,  -81,  -78,  -76,  -73,
		 -71,  -68,  -65,  -63,  -60,  -57,  -54,  -51,
		 -49,  -46,  -43,  -40,  -37,  -34,  -31,  -28,
		 -25,  -22,  -19,  -16,  -12,   -9,   -6,   -3, 0};
#endif

/*
 * Sine data in 16 bit format.
 * -128..0..127 ~ -Pi..0..Pi/256*255
 */
#ifndef __NO_SIN_TABLE16__
	const int16 Sin_Table16[257]  = {
		     0,    804,   1608,   2410,   3212,   4011,   4808,   5602,
		  6393,   7179,   7962,   8739,   9512,  10278,  11039,  11793,
		 12539,  13279,  14010,  14732,  15446,  16151,  16846,  17530,
		 18204,  18868,  19519,  20159,  20787,  21403,  22005,  22594,
		 23170,  23731,  24279,  24811,  25329,  25832,  26319,  26790,
		 27245,  27683,  28105,  28510,  28898,  29268,  29621,  29956,
		 30273,  30571,  30852,  31113,  31356,  31580,  31785,  31971,
		 32137,  32285,  32412,  32521,  32609,  32678,  32728,  32757,
		 32767,  32757,  32728,  32678,  32609,  32521,  32412,  32285,
		 32137,  31971,  31785,  31580,  31356,  31113,  30852,  30571,
		 30273,  29956,  29621,  29268,  28898,  28510,  28105,  27683,
		 27245,  26790,  26319,  25832,  25329,  24811,  24279,  23731,
		 23170,  22594,  22005,  21403,  20787,  20159,  19519,  18868,
		 18204,  17530,  16846,  16151,  15446,  14732,  14010,  13279,
		 12539,  11793,  11039,  10278,   9512,   8739,   7962,   7179,
		  6393,   5602,   4808,  4011,    3212,   2410,   1608,    804,
		     0,   -804,  -1608,  -2410,  -3212,  -4011,  -4808,  -5602,
		 -6393,  -7179,  -7962, -8739,   -9512, -10278, -11039, -11793,
		-12539, -13279, -14010, -14732, -15446, -16151, -16846, -17530,
		-18204, -18868, -19519, -20159, -20787, -21403, -22005, -22594,
		-23170, -23731, -24279, -24811, -25329, -25832, -26319, -26790,
		-27245, -27683, -28105, -28510, -28898, -29268, -29621, -29956,
		-30273, -30571, -30852, -31113, -31356, -31580, -31785, -31971,
		-32137, -32285, -32412, -32521, -32609, -32678, -32728, -32757,
		-32767, -32757, -32728, -32678, -32609, -32521, -32412, -32285,
		-32137, -31971, -31785, -31580, -31356, -31113, -30852, -30571,
		-30273, -29956, -29621, -29268, -28898, -28510, -28105, -27683,
		-27245, -26790, -26319, -25832, -25329, -24811, -24279, -23731,
		-23170, -22594, -22005, -21403, -20787, -20159, -19519, -18868,
		-18204, -17530, -16846, -16151, -15446, -14732, -14010, -13279,
		-12539, -11793, -11039, -10278, -9512,   -8739,  -7962,  -7179,
		 -6393,  -5602,  -4808,  -4011, -3212,   -2410,  -1608,   -804, 0}; 
#endif
	
/*
 * Sine data in 32 bit format.
 * -128..0..127 ~ -Pi..0..Pi/256*255
 */
#ifndef __NO_SIN_TABLE32__
	const int32 Sin_Table32[257]  = {
		          0,    52701887,   105372028,   157978697,   210490206,   262874923,   315101294,   367137860,
		  418953276,   470516330,   521795963,   572761285,   623381597,   673626408,   723465451,   772868706,
		  821806413,   870249095,   918167571,   965532978,  1012316784,  1058490807,  1104027236,  1148898640,
		 1193077990,  1236538675,  1279254515,  1321199780,  1362349204,  1402677999,  1442161874,  1480777044,
		 1518500249,  1555308767,  1591180425,  1626093615,  1660027308,  1692961061,  1724875039,  1755750016,
		 1785567395,  1814309215,  1841958164,  1868497585,  1893911493,  1918184580,  1941302224,  1963250500,
		 1984016188,  2003586778,  2021950483,  2039096240,  2055013722,  2069693341,  2083126253,  2095304369,
		 2106220351,  2115867625,  2124240379,  2131333571,  2137142926,  2141664947,  2144896909,  2146836865,
		 2147483647,  2146836865,  2144896909,  2141664947,  2137142926,  2131333571,  2124240379,  2115867625,
		 2106220351,  2095304369,  2083126253,  2069693341,  2055013722,  2039096240,  2021950483,  2003586778,
		 1984016188,  1963250500,  1941302224,  1918184580,  1893911493,  1868497585,  1841958164,  1814309215,
		 1785567395,  1755750016,  1724875039,  1692961061,  1660027308,  1626093615,  1591180425,  1555308767,
		 1518500249,  1480777044,  1442161874,  1402677999,  1362349204,  1321199780,  1279254515,  1236538675,
		 1193077990,  1148898640,  1104027236,  1058490807,  1012316784,   965532978,   918167571,   870249095,
		  821806413,  772868706,    723465451,   673626408,   623381597,   572761285,   521795963,   470516330,
		  418953276,  367137860,    315101294,   262874923,   210490206,   157978697,   105372028,    52701887,
		          0,   -52701887,  -105372028,  -157978697,  -210490206,  -262874923,  -315101294,  -367137860,
		 -418953276,  -470516330,  -521795963,  -572761285,  -623381597,  -673626408,  -723465451,  -772868706,
		 -821806413,  -870249095,  -918167571,  -965532978, -1012316784, -1058490807, -1104027236, -1148898640,
		-1193077990, -1236538675, -1279254515, -1321199780, -1362349204, -1402677999, -1442161874, -1480777044,
		-1518500249, -1555308767, -1591180425, -1626093615, -1660027308, -1692961061, -1724875039, -1755750016,
		-1785567395, -1814309215, -1841958164, -1868497585, -1893911493, -1918184580, -1941302224, -1963250500,
		-1984016188, -2003586778, -2021950483, -2039096240, -2055013722, -2069693341, -2083126253, -2095304369,
		-2106220351, -2115867625, -2124240379, -2131333571, -2137142926, -2141664947, -2144896909, -2146836865,
		-2147483647, -2146836865, -2144896909, -2141664947, -2137142926, -2131333571, -2124240379, -2115867625,
		-2106220351, -2095304369, -2083126253, -2069693341, -2055013722, -2039096240, -2021950483, -2003586778,
		-1984016188, -1963250500, -1941302224, -1918184580, -1893911493, -1868497585, -1841958164, -1814309215,
		-1785567395, -1755750016, -1724875039, -1692961061, -1660027308, -1626093615, -1591180425, -1555308767,
		-1518500249, -1480777044, -1442161874, -1402677999, -1362349204, -1321199780, -1279254515, -1236538675,
		-1193077990, -1148898640, -1104027236, -1058490807, -1012316784,  -965532978,  -918167571,  -870249095,
		 -821806413,  -772868706,  -723465451,  -673626408,  -623381597,  -572761285,  -521795963,  -470516330,
		 -418953276,  -367137860,  -315101294,  -262874923,  -210490206,  -157978697,  -105372028,   -52701887, 0};
#endif

#ifndef __NO_ASIN_TABLE8__
/* FiP8 arcsine(x) lookup table data for x = 0..1 */
const int8 Asin_Table8[257] = {
   0,    0,    0,    0,    1,    1,    1,    1,
   1,    1,    2,    2,    2,    2,    2,    2,
   3,    3,    3,    3,    3,    3,    4,    4,
   4,    4,    4,    4,    4,    5,    5,    5,
   5,    5,    5,    6,    6,    6,    6,    6,
   6,    7,    7,    7,    7,    7,    7,    8,
   8,    8,    8,    8,    8,    8,    9,    9,
   9,    9,    9,    9,   10,   10,   10,   10,
  10,   10,   11,   11,   11,   11,   11,   11,
  12,   12,   12,   12,   12,   12,   13,   13,
  13,   13,   13,   13,   14,   14,   14,   14,
  14,   14,   15,   15,   15,   15,   15,   15,
  16,   16,   16,   16,   16,   17,   17,   17,
  17,   17,   17,   18,   18,   18,   18,   18,
  18,   19,   19,   19,   19,   19,   20,   20,
  20,   20,   20,   20,   21,   21,   21,   21,
  21,   22,   22,   22,   22,   22,   22,   23,
  23,   23,   23,   23,   24,   24,   24,   24,
  24,   25,   25,   25,   25,   25,   26,   26,
  26,   26,   26,   26,   27,   27,   27,   27,
  28,   28,   28,   28,   28,   29,   29,   29,
  29,   29,   30,   30,   30,   30,   30,   31,
  31,   31,   31,   32,   32,   32,   32,   32,
  33,   33,   33,   33,   34,   34,   34,   34,
  35,   35,   35,   35,   36,   36,   36,   36,
  37,   37,   37,   37,   38,   38,   38,   38,
  39,   39,   39,   39,   40,   40,   40,   41,
  41,   41,   42,   42,   42,   42,   43,   43,
  43,   44,   44,   44,   45,   45,   45,   46,
  46,   47,   47,   47,   48,   48,   49,   49,
  50,   50,   50,   51,   51,   52,   53,   53,
  54,   54,   55,   56,   57,   58,   59,   60,
  64};
#endif
   
#ifndef __NO_ASIN_TABLE16__   
/* FiP16 arcsine(x) lookup table data for x = 0..1 */
const int16 Asin_Table16[257] = {
     0,     41,     81,    122,    163,    204,    244,    285,
   326,    367,    408,    448,    489,    530,    571,    612,
   652,    693,    734,    775,    816,    857,    897,    938,
   979,   1020,   1061,   1102,   1143,   1184,   1225,   1266,
  1307,   1348,   1389,   1431,   1472,   1513,   1554,   1595,
  1636,   1678,   1719,   1760,   1802,   1843,   1884,   1926,
  1967,   2009,   2050,   2092,   2134,   2175,   2217,   2259,
  2300,   2342,   2384,   2426,   2468,   2510,   2551,   2593,
  2636,   2678,   2720,   2762,   2804,   2847,   2889,   2931,
  2974,   3016,   3059,   3101,   3144,   3187,   3229,   3272,
  3315,   3358,   3401,   3444,   3487,   3530,   3573,   3617,
  3660,   3704,   3747,   3791,   3834,   3878,   3922,   3965,
  4009,   4053,   4097,   4142,   4186,   4230,   4275,   4319,
  4364,   4408,   4453,   4498,   4543,   4588,   4633,   4678,
  4723,   4768,   4814,   4859,   4905,   4951,   4997,   5043,
  5089,   5135,   5181,   5228,   5274,   5321,   5367,   5414,
  5461,   5508,   5556,   5603,   5651,   5698,   5746,   5794,
  5842,   5890,   5938,   5987,   6035,   6084,   6133,   6182,
  6231,   6281,   6330,   6380,   6430,   6480,   6530,   6580,
  6631,   6681,   6732,   6783,   6835,   6886,   6938,   6990,
  7042,   7094,   7147,   7199,   7252,   7306,   7359,   7413,
  7466,   7521,   7575,   7630,   7684,   7740,   7795,   7851,
  7907,   7963,   8019,   8076,   8133,   8191,   8249,   8307,
  8365,   8424,   8483,   8543,   8602,   8663,   8723,   8784,
  8846,   8907,   8970,   9032,   9095,   9159,   9223,   9288,
  9353,   9418,   9484,   9551,   9618,   9686,   9754,   9823,
  9892,   9963,  10034,  10105,  10177,  10251,  10324,  10399,
 10475,  10551,  10628,  10706,  10785,  10866,  10947,  11029,
 11113,  11198,  11284,  11371,  11460,  11550,  11642,  11736,
 11831,  11929,  12028,  12130,  12234,  12340,  12449,  12561,
 12677,  12796,  12919,  13046,  13178,  13315,  13459,  13610,
 13770,  13939,  14121,  14319,  14538,  14786,  15079,  15462,
 16384};
#endif
	 
#ifndef __NO_ASIN_TABLE32__	 
/* FiP32 arcsine(x) lookup table data for x = 0..1 */
const int32 Asin_Table32[257] = {
          0,     2670184,     5340408,     8010714,    10681142,    13351733,    16022528,    18693568,
   21364893,    24036545,    26708564,    29380991,    32053868,    34727235,    37401135,    40075607,
   42750693,    45426435,    48102875,    50780053,    53458012,    56136793,    58816439,    61496991,
   64178491,    66860983,    69544508,    72229108,    74914828,    77601709,    80289795,    82979128,
   85669753,    88361714,    91055052,    93749814,    96446043,    99143783,   101843079,   104543976,
  107246519,   109950753,   112656724,   115364478,   118074061,   120785519,   123498899,   126214247,
  128931612,   131651040,   134372580,   137096279,   139822187,   142550351,   145280822,   148013648,
  150748879,   153486567,   156226761,   158969512,   161714873,   164462894,   167213628,   169967129,
  172723448,   175482640,   178244759,   181009860,   183777996,   186549225,   189323602,   192101183,
  194882027,   197666190,   200453730,   203244708,   206039182,   208837211,   211638858,   214444183,
  217253248,   220066115,   222882849,   225703513,   228528172,   231356891,   234189736,   237026775,
  239868075,   242713705,   245563734,   248418233,   251277272,   254140923,   257009260,   259882357,
  262760287,   265643127,   268530954,   271423845,   274321879,   277225137,   280133698,   283047646,
  285967063,   288892034,   291822645,   294758982,   297701134,   300649190,   303603242,   306563381,
  309529700,   312502296,   315481265,   318466704,   321458714,   324457396,   327462853,   330475188,
  333494510,   336520925,   339554544,   342595479,   345643843,   348699752,   351763325,   354834681,
  357913941,   361001232,   364096678,   367200410,   370312558,   373433256,   376562641,   379700852,
  382848031,   386004322,   389169873,   392344834,   395529360,   398723605,   401927731,   405141901,
  408366282,   411601043,   414846360,   418102409,   421369374,   424647439,   427936794,   431237635,
  434550160,   437874573,   441211081,   444559900,   447921246,   451295344,   454682423,   458082719,
  461496472,   464923930,   468365347,   471820982,   475291102,   478775983,   482275905,   485791159,
  489322041,   492868857,   496431923,   500011561,   503608104,   507221896,   510853289,   514502647,
  518170345,   521856769,   525562319,   529287405,   533032452,   536797899,   540584200,   544391822,
  548221250,   552072985,   555947547,   559845473,   563767319,   567713663,   571685104,   575682264,
  579705789,   583756348,   587834641,   591941392,   596077357,   600243323,   604440112,   608668579,
  612929618,   617224164,   621553193,   625917727,   630318836,   634757644,   639235328,   643753126,
  648312337,   652914330,   657560548,   662252512,   666991828,   671780194,   676619409,   681511377,
  686458121,   691461793,   696524681,   701649228,   706838041,   712093912,   717419832,   722819017,
  728294928,   733851298,   739492166,   745221912,   751045296,   756967509,   762994231,   769131691,
  775386752,   781767001,   788280861,   794937727,   801748130,   808723935,   815878600,   823227480,
  830788232,   838581326,   846630714,   854964718,   863617241,   872629428,   882052043,   891948935,
  902402317,   913521122,   925455010,   938419510,   952745516,   968990224,   988240438,  1013302934,
 1073741824};
#endif


#ifndef __NO_ACOS_TABLE8__
/* FiP8 arccos(x) lookup table data for x = 0..1 */
const int8 Acos_Table8[257] = {
  64,   64,   64,   64,   63,   63,   63,   63,
  63,   63,   62,   62,   62,   62,   62,   62,
  61,   61,   61,   61,   61,   61,   60,   60,
  60,   60,   60,   60,   60,   59,   59,   59,
  59,   59,   59,   58,   58,   58,   58,   58,
  58,   57,   57,   57,   57,   57,   57,   56,
  56,   56,   56,   56,   56,   56,   55,   55,
  55,   55,   55,   55,   54,   54,   54,   54,
  54,   54,   53,   53,   53,   53,   53,   53,
  52,   52,   52,   52,   52,   52,   51,   51,
  51,   51,   51,   51,   50,   50,   50,   50,
  50,   50,   49,   49,   49,   49,   49,   49,
  48,   48,   48,   48,   48,   47,   47,   47,
  47,   47,   47,   46,   46,   46,   46,   46,
  46,   45,   45,   45,   45,   45,   44,   44,
  44,   44,   44,   44,   43,   43,   43,   43,
  43,   42,   42,   42,   42,   42,   42,   41,
  41,   41,   41,   41,   40,   40,   40,   40,
  40,   39,   39,   39,   39,   39,   38,   38,
  38,   38,   38,   38,   37,   37,   37,   37,
  36,   36,   36,   36,   36,   35,   35,   35,
  35,   35,   34,   34,   34,   34,   34,   33,
  33,   33,   33,   32,   32,   32,   32,   32,
  31,   31,   31,   31,   30,   30,   30,   30,
  29,   29,   29,   29,   28,   28,   28,   28,
  27,   27,   27,   27,   26,   26,   26,   26,
  25,   25,   25,   25,   24,   24,   24,   23,
  23,   23,   22,   22,   22,   22,   21,   21,
  21,   20,   20,   20,   19,   19,   19,   18,
  18,   17,   17,   17,   16,   16,   15,   15,
  14,   14,   14,   13,   13,   12,   11,   11,
  10,   10,    9,    8,    7,    6,    5,    4,
   0};
#endif

#ifndef __NO_ACOS_TABLE16__  
/* FiP16 arccos(x) lookup table data for x = 0..1 */
const int16 Acos_Table16[257] = {
 16384,  16343,  16303,  16262,  16221,  16180,  16140,  16099,
 16058,  16017,  15976,  15936,  15895,  15854,  15813,  15772,
 15732,  15691,  15650,  15609,  15568,  15527,  15487,  15446,
 15405,  15364,  15323,  15282,  15241,  15200,  15159,  15118,
 15077,  15036,  14995,  14953,  14912,  14871,  14830,  14789,
 14748,  14706,  14665,  14624,  14582,  14541,  14500,  14458,
 14417,  14375,  14334,  14292,  14250,  14209,  14167,  14125,
 14084,  14042,  14000,  13958,  13916,  13874,  13833,  13791,
 13748,  13706,  13664,  13622,  13580,  13537,  13495,  13453,
 13410,  13368,  13325,  13283,  13240,  13197,  13155,  13112,
 13069,  13026,  12983,  12940,  12897,  12854,  12811,  12767,
 12724,  12680,  12637,  12593,  12550,  12506,  12462,  12419,
 12375,  12331,  12287,  12242,  12198,  12154,  12109,  12065,
 12020,  11976,  11931,  11886,  11841,  11796,  11751,  11706,
 11661,  11616,  11570,  11525,  11479,  11433,  11387,  11341,
 11295,  11249,  11203,  11156,  11110,  11063,  11017,  10970,
 10923,  10876,  10828,  10781,  10733,  10686,  10638,  10590,
 10542,  10494,  10446,  10397,  10349,  10300,  10251,  10202,
 10153,  10103,  10054,  10004,   9954,   9904,   9854,   9804,
  9753,   9703,   9652,   9601,   9549,   9498,   9446,   9394,
  9342,   9290,   9237,   9185,   9132,   9078,   9025,   8971,
  8918,   8863,   8809,   8754,   8700,   8644,   8589,   8533,
  8477,   8421,   8365,   8308,   8251,   8193,   8135,   8077,
  8019,   7960,   7901,   7841,   7782,   7721,   7661,   7600,
  7538,   7477,   7414,   7352,   7289,   7225,   7161,   7096,
  7031,   6966,   6900,   6833,   6766,   6698,   6630,   6561,
  6492,   6421,   6350,   6279,   6207,   6133,   6060,   5985,
  5909,   5833,   5756,   5678,   5599,   5518,   5437,   5355,
  5271,   5186,   5100,   5013,   4924,   4834,   4742,   4648,
  4553,   4455,   4356,   4254,   4150,   4044,   3935,   3823,
  3707,   3588,   3465,   3338,   3206,   3069,   2925,   2774,
  2614,   2445,   2263,   2065,   1846,   1598,   1305,    922,
     0};
#endif

#ifndef __NO_ACOS_TABLE32__ 
/* FiP32 arccos(x) lookup table data for x = 0..1 */
const int32 Acos_Table32[257] = {
 1073741824,  1071071640,  1068401416,  1065731110,  1063060682,  1060390091,  1057719296,  1055048256,
 1052376931,  1049705279,  1047033260,  1044360833,  1041687956,  1039014589,  1036340689,  1033666217,
 1030991131,  1028315389,  1025638949,  1022961771,  1020283812,  1017605031,  1014925385,  1012244833,
 1009563333,  1006880841,  1004197316,  1001512716,   998826996,   996140115,   993452029,   990762696,
  988072071,   985380110,   982686772,   979992010,   977295781,   974598041,   971898745,   969197848,
  966495305,   963791071,   961085100,   958377346,   955667763,   952956305,   950242925,   947527577,
  944810212,   942090784,   939369244,   936645545,   933919637,   931191473,   928461002,   925728176,
  922992945,   920255257,   917515063,   914772312,   912026951,   909278930,   906528196,   903774695,
  901018376,   898259184,   895497065,   892731964,   889963828,   887192599,   884418222,   881640641,
  878859797,   876075634,   873288094,   870497116,   867702642,   864904613,   862102966,   859297641,
  856488576,   853675709,   850858975,   848038311,   845213652,   842384933,   839552088,   836715049,
  833873749,   831028119,   828178090,   825323591,   822464552,   819600901,   816732564,   813859467,
  810981537,   808098697,   805210870,   802317979,   799419945,   796516687,   793608126,   790694178,
  787774761,   784849790,   781919179,   778982842,   776040690,   773092634,   770138582,   767178443,
  764212124,   761239528,   758260559,   755275120,   752283110,   749284428,   746278971,   743266636,
  740247314,   737220899,   734187280,   731146345,   728097981,   725042072,   721978499,   718907143,
  715827883,   712740592,   709645146,   706541414,   703429266,   700308568,   697179183,   694040972,
  690893793,   687737502,   684571951,   681396990,   678212464,   675018219,   671814093,   668599923,
  665375542,   662140781,   658895464,   655639415,   652372450,   649094385,   645805030,   642504189,
  639191664,   635867251,   632530743,   629181924,   625820578,   622446480,   619059401,   615659105,
  612245352,   608817894,   605376477,   601920842,   598450722,   594965841,   591465919,   587950665,
  584419783,   580872967,   577309901,   573730263,   570133720,   566519928,   562888535,   559239177,
  555571479,   551885055,   548179505,   544454419,   540709372,   536943925,   533157624,   529350002,
  525520574,   521668839,   517794277,   513896351,   509974505,   506028161,   502056720,   498059560,
  494036035,   489985476,   485907183,   481800432,   477664467,   473498501,   469301712,   465073245,
  460812206,   456517660,   452188631,   447824097,   443422988,   438984180,   434506496,   429988698,
  425429487,   420827494,   416181276,   411489312,   406749996,   401961630,   397122415,   392230447,
  387283703,   382280031,   377217143,   372092596,   366903783,   361647912,   356321992,   350922807,
  345446896,   339890526,   334249658,   328519912,   322696528,   316774315,   310747593,   304610133,
  298355072,   291974823,   285460963,   278804097,   271993694,   265017889,   257863224,   250514344,
  242953592,   235160498,   227111110,   218777106,   210124583,   201112396,   191689781,   181792889,
  171339507,   160220702,   148286814,   135322314,   120996308,   104751600,    85501386,    60438890,
          0};
 #endif
