// Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2465 $
// $LastChangedDate:: 2022-02-15 15:12:58 +0100#$
//
// Creates interface function for Xcos.
// Parameters:
// libName - Library name
// block - Java X2C block object
// varargin - (optional) project root directory for external blocks

function [] = createIFunction(libName, block, varargin)

//------------------------------------------------------------------------------
// Get relevant block information
//------------------------------------------------------------------------------
if length(varargin) == 0 then
    // internal library
    blockInfo = getBlockInfo(libName, block);
else
    // external library
    blockInfo = getBlockInfo(libName, block, varargin(1));
end

//------------------------------------------------------------------------------
// Create file
//------------------------------------------------------------------------------
try
    // open/create file
    fileName = blockInfo.scilabDir + blockInfo.xcosName + ".sci";
    fileID = mopen(fileName, "wt");

    // print header
    time = datevec(datenum());
    mfprintf(fileID, "// This file was generated by %s on %02d-%02d-%04d %02d:%02d\n\n", "createIFunction.sci", time(3), time(2), time(1), time(4), time(5));
    // TODO: expand file header (e.g. with license)

    mfprintf(fileID, "function [x_sci,y_sci,typ_sci] = %s(job, arg1, arg2)\n", blockInfo.xcosName);
    mfprintf(fileID, "\n");
    mfprintf(fileID, "thisBlockData.Library = '"%s'";\n", libName);
    mfprintf(fileID, "thisBlockData.block_type_name = '"%s'";\n", blockInfo.blockName);
    mfprintf(fileID, "thisBlockData.block_xcosfilename = '"%s'";\n", blockInfo.xcosName);
    mfprintf(fileID, "\n");
    mfprintf(fileID, "jimport('"at.lcm.x2c.parameterEditor.X2cParameterEditor_Xcos'");\n");
    mfprintf(fileID, "jimport('"at.lcm.x2c.utils.Utils'");\n");
    mfprintf(fileID, "\n");
    mfprintf(fileID, "x_sci = [];\n");
    mfprintf(fileID, "y_sci = [];\n");
    mfprintf(fileID, "typ_sci = [];\n");
    mfprintf(fileID, "X2CRootDir = jinvoke(jinvoke(Utils, '"getRootDirectory'"), '"toString'");\n");
    mfprintf(fileID, "X2CRootDir = strsubst(X2CRootDir, '"\\'", '"/'");\n");
    
    mfprintf(fileID, "\n");
    mfprintf(fileID, "select job\n");

    mfprintf(fileID, "case '"plot'" then\n");
    mfprintf(fileID, "// ++ BlockGenerator: Plot Section\n");
    if ~isempty(blockInfo.Visualization.PlotSection) then
        // TODO: is this all necessary? wouldn't last line be sufficient?
        mfprintf(fileID, "    block_graphics = arg1.graphics;\n");
        mfprintf(fileID, "    block_model = arg1.model;\n");
        mfprintf(fileID, "    exprs = block_graphics.exprs;\n");
        mfprintf(fileID, "    xcosBlockId = block_model.uid;\n");
        mfprintf(fileID, "    if exists('"X2C_sampleTime'") then\n");
        mfprintf(fileID, "        X2C_sampleTime_tmp = X2C_sampleTime;\n");
        mfprintf(fileID, "    else\n");
        mfprintf(fileID, "        X2C_sampleTime_tmp = -1;\n");
        mfprintf(fileID, "    end\n");
        mfprintf(fileID, "    x2cPEdit = jnewInstance(X2cParameterEditor_Xcos, exprs, xcosBlockId, X2C_sampleTime_tmp);\n");
        mfprintf(fileID, "    %s\n", blockInfo.Visualization.PlotSection);
    else
        mfprintf(fileID, "    standard_draw(arg1);\n");
    end
    mfprintf(fileID, "// -- BlockGenerator: Plot Section\n");
    mfprintf(fileID, "\n");

    mfprintf(fileID, "case '"getinputs'" then\n");
    mfprintf(fileID, "    [x_sci,y_sci,typ_sci] = standard_inputs(arg1);\n");
    mfprintf(fileID, "\n");
    
    mfprintf(fileID, "case '"getoutputs'" then\n");
    mfprintf(fileID, "    [x_sci,y_sci,typ_sci] = standard_outputs(arg1);\n");
    mfprintf(fileID, "\n");
    
    mfprintf(fileID, "case '"getorigin'" then\n");
    mfprintf(fileID, "    [x_sci,y_sci] = standard_origin(arg1);\n");
    mfprintf(fileID, "\n");
    
    // *** SET PROECEDURE ***
    mfprintf(fileID, "case '"set'" then\n");
    mfprintf(fileID, "    x_sci = arg1;\n");
    mfprintf(fileID, "    block_graphics = arg1.graphics;\n");
    mfprintf(fileID, "    block_model = arg1.model;\n");
    mfprintf(fileID, "    exprs = block_graphics.exprs;\n");
    mfprintf(fileID, "    xcosBlockId = block_model.uid;\n");
    mfprintf(fileID, "    while %%t do\n");
    mfprintf(fileID, "        if exists('"X2C_sampleTime'") then\n");
    mfprintf(fileID, "            X2C_sampleTime_tmp = X2C_sampleTime;\n");
    mfprintf(fileID, "        else\n");
    mfprintf(fileID, "            X2C_sampleTime_tmp = -1;\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        x2cPEdit = jnewInstance(X2cParameterEditor_Xcos, exprs, xcosBlockId, X2C_sampleTime_tmp);\n");
    mfprintf(fileID, "        thisBlock_isPartOfModel = jinvoke(x2cPEdit, '"isBlockPartOfModel'");\n");
    mfprintf(fileID, "        if (~ exists('"X2C_sampleTime'")) & (~ thisBlock_isPartOfModel) then\n");
    mfprintf(fileID, "            warning('"X2C_sampleTime is not set; default is used'");\n");
    mfprintf(fileID, "        elseif (exists('"X2C_sampleTime'")) & (thisBlock_isPartOfModel) then\n");
    mfprintf(fileID, "            warning('"X2C_sampleTime is set; but model sampleTime is used'");\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        if ~ exists('"%%scicos_prob'") then\n");
    mfprintf(fileID, "            // double click on icon\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "            ok = jinvoke(x2cPEdit, '"runGUI'");\n");
    mfprintf(fileID, "        else\n");
    mfprintf(fileID, "            ok = %%t;\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        if ~ok then\n");
    mfprintf(fileID, "            break;\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        used_impl = jinvoke(x2cPEdit, '"getUsedImplementationName'");\n");
    mfprintf(fileID, "        block_model.sim = list('"%s_'"  + used_impl + '"_C'", 4);\n", blockInfo.xcosName);
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        xcp = jinvoke(x2cPEdit, '"getControlerParameter'");\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        block_model.ipar = double(jinvoke(xcp, '"getIntegerParameter_values'"));\n");
    mfprintf(fileID, "        block_model.rpar = double(jinvoke(xcp, '"getDoubleParameter_values'"));\n");
    mfprintf(fileID, "        po = jinvoke(x2cPEdit, '"getControlerParameter_DoubleArrayParameter_values'");\n");
    mfprintf(fileID, "        otab = list();\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        for i=0:jinvoke(po, '"size'")-1\n");
    mfprintf(fileID, "            otab($+1) = jinvoke(po, '"get'", i);\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        block_model.opar = otab;\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        exprs = jinvoke(x2cPEdit, '"getXcosExprs'")'';\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        block_graphics.exprs = exprs;\n");
    mfprintf(fileID, "        x_sci.graphics = block_graphics;\n");
    mfprintf(fileID, "        x_sci.model = block_model;\n");
    
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "// ++ BlockGenerator: Define Section\n");
    if ~isempty(blockInfo.Visualization.DefineSection) then
        define_section_string = strsubst(blockInfo.Visualization.DefineSection, "x_sci.graphics.style", "define_style");
        define_section_string = strsubst(define_section_string, "x_sci.graphics.in_style", "define_in_style");
        define_section_string = strsubst(define_section_string, "x_sci.graphics.out_style", "define_out_style");
        mfprintf(fileID, "%s\n", define_section_string);
    end
    mfprintf(fileID, "// -- BlockGenerator: Define Section\n");
    mfprintf(fileID, "    \n");
    
    // *** SCILAB 6 UPDATE
    mfprintf(fileID, "        if isdef('"define_style'", '"l'")\n");
    mfprintf(fileID, "            define_style_struct = string2struct(define_style);\n");
    mfprintf(fileID, "            if isfield(define_style_struct, '"image'")\n");
    mfprintf(fileID, "                image_link = getStyleImageLink(thisBlockData.Library, thisBlockData.block_type_name, define_style_struct.image)\n");
    mfprintf(fileID, "                define_style_struct.image = image_link;\n");
    mfprintf(fileID, "            end\n");
    mfprintf(fileID, "            style_string = struct2string(define_style_struct);\n");
    mfprintf(fileID, "            x_sci.graphics.style = thisBlockData.block_xcosfilename + '";'" + style_string;\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        \n");
    
    mfprintf(fileID, "        break;\n");
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "    \n");

    // *** DEFINE PROCEDURE ***
    mfprintf(fileID, "case '"define'" then\n");
    mfprintf(fileID, "    if exists('"X2C_sampleTime'") then\n");
    mfprintf(fileID, "        X2C_sampleTime_tmp = X2C_sampleTime;\n");
    mfprintf(fileID, "    else\n");
    mfprintf(fileID, "        X2C_sampleTime_tmp = -1;\n");
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "    x2cPEdit = jnewInstance(X2cParameterEditor_Xcos, thisBlockData.block_type_name, X2C_sampleTime_tmp);\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    inport_labels = jinvoke(x2cPEdit, '"getBlockInportNames'");\n");
    mfprintf(fileID, "    outport_labels = jinvoke(x2cPEdit, '"getBlockOutportNames'");\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    // see: help scicos_model\n");
    mfprintf(fileID, "    model = scicos_model();\n");
    mfprintf(fileID, "    used_impl = jinvoke(x2cPEdit, '"getUsedImplementationName'");\n");
    mfprintf(fileID, "    model.sim = list('"%s_'"  + used_impl + '"_C'", 4);\n", blockInfo.xcosName);
    mfprintf(fileID, "    model.in = jinvoke(x2cPEdit, '"getBlockInport_xcosDef'");\n");
    if  blockInfo.TimeDependency then
        mfprintf(fileID, "    model.evtin = [1];\n");
    else
        mfprintf(fileID, "    model.evtin = [];\n");
    end
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    model.out = jinvoke(x2cPEdit, '"getBlockOutport_xcosDef'");\n");
    mfprintf(fileID, "    model.state = [];   // continuous states\n");
    mfprintf(fileID, "    model.dstate = [];  // discrete-time states\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    model.ipar = double(jinvoke(x2cPEdit, '"getControlerParameter_IntegerParameter_values'")'');\n");
    mfprintf(fileID, "    model.rpar = double(jinvoke(x2cPEdit, '"getControlerParameter_DoubleParameter_values'")'');\n");
    mfprintf(fileID, "    model.blocktype = '"c'";\n");
    mfprintf(fileID, "    model.nmode = 0;\n");
    mfprintf(fileID, "    model.nzcross = 0;\n");
    if blockInfo.DirectFeedThrough then
        mfprintf(fileID, "    model.dep_ut = [%%t %%f];\n");
    else
        mfprintf(fileID, "    model.dep_ut = [%%f %%f];\n");
    end
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    exprs = jinvoke(x2cPEdit, '"getXcosExprs'")'';\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "// ++ BlockGenerator: BlockIconDefaultSize\n");
    if ~isempty(blockInfo.Visualization.BlockIconDefaultSize) then
        mfprintf(fileID, "    blockIconDefaultSize = %s;\n", blockInfo.Visualization.BlockIconDefaultSize);
    else
        mfprintf(fileID, "    blockIconDefaultSize = %s;\n", ["[5, " + string(2 + blockInfo.maxPorts) + "]"]);
    end
    mfprintf(fileID, "// -- BlockGenerator: BlockIconDefaultSize\n");
    mfprintf(fileID, "    \n");

    mfprintf(fileID, "    gr_i = [];\n")
    mfprintf(fileID, "    \n");
    
    mfprintf(fileID, "    x_sci = standard_define(blockIconDefaultSize, model, exprs, gr_i);\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    // see help: graphics structure\n");
    mfprintf(fileID, "    x_sci.graphics.in_label = inport_labels; // labels of regular input ports\n");
    mfprintf(fileID, "    x_sci.graphics.out_label = outport_labels; // labels of regular output ports\n");
    
    // *** default block color = blue; default style = non-rounded corners
    mfprintf(fileID, "    style = struct();\n");
    mfprintf(fileID, "    style.fillColor = '"#61BDFC'";\n");
    mfprintf(fileID, "    style.gradientColor = '"#61BDFC'";\n");
    mfprintf(fileID, "    style.rounded = '"false'";\n");
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "    define_in_style = x_sci.graphics.in_style;\n");
    mfprintf(fileID, "    define_out_style = x_sci.graphics.out_style;\n");
    
    mfprintf(fileID, "    \n");
    mfprintf(fileID, "// ++ BlockGenerator: Define Section\n");
    if ~isempty(blockInfo.Visualization.DefineSection) then
        // replace direct access to use temporary variables to allow modification after inserting Block Generator 'Define section'
        define_sect_subst = strsubst(blockInfo.Visualization.DefineSection, "x_sci.graphics.style", "define_style");
        define_sect_subst = strsubst(define_sect_subst, "x_sci.graphics.in_style", "define_in_style");
        define_sect_subst = strsubst(define_sect_subst, "x_sci.graphics.out_style", "define_out_style");
        mfprintf(fileID, "%s\n", define_sect_subst);
    end
    mfprintf(fileID, "// -- BlockGenerator: Define Section\n");
    mfprintf(fileID, "    \n");
	
    // *** add structure members from define style structure into final style structure
    mfprintf(fileID, "    if isdef('"define_style'", '"l'")\n");
    mfprintf(fileID, "        define_style_struct = string2struct(define_style);\n");
    mfprintf(fileID, "        style = mergeStructures(style, define_style_struct);\n");
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "    \n");  
    
    // *** add Block Generator 'Style' section
    mfprintf(fileID, "// ++ BlockGenerator: Style Section\n");
    style_sect_subst = strsubst(blockInfo.Visualization.Style, "style.", "style_tmp.");
    style_sect_subst = strsubst(style_sect_subst, "curDir", "pwd()");
    if ~isempty(style_sect_subst) then
        mfprintf(fileID, "%s\n", style_sect_subst);
    end
    mfprintf(fileID, "// -- BlockGenerator: Style Section\n");
    mfprintf(fileID, "    \n");

    // *** processing 'Style' section
    mfprintf(fileID, "    if isdef('"style_tmp'", '"l'")\n");
    mfprintf(fileID, "        // processing image member\n");
    mfprintf(fileID, "        if isfield(style_tmp, '"image'")\n");
    mfprintf(fileID, "            style_tmp.image = getStyleImageLink(thisBlockData.Library, thisBlockData.block_type_name, style_tmp.image, %%t);\n");
    mfprintf(fileID, "            if isempty(style_tmp.image)\n");
    mfprintf(fileID, "                // removes image member if ''Dummy.svg'' was used before\n");
    mfprintf(fileID, "                style_tmp.image = null();\n");
    mfprintf(fileID, "            end\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "        \n");
    mfprintf(fileID, "        // add style structure fields to style structure\n");
    mfprintf(fileID, "        style = mergeStructures(style, style_tmp);\n");
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "    \n");
    
    // *** default image location if no explicit image location is defined
    mfprintf(fileID, "    // try default image location if no explicit image location is defined\n");
    mfprintf(fileID, "    if ~isfield(style, '"image'")\n");
    mfprintf(fileID, "        image_link = getStyleImageLink(thisBlockData.Library, thisBlockData.block_type_name);\n");
    mfprintf(fileID, "        if ~isempty(image_link)\n");
    mfprintf(fileID, "            style.image = image_link;\n");
    mfprintf(fileID, "        end\n");
    mfprintf(fileID, "    end\n");
    mfprintf(fileID, "    \n");
    
	// *** convert style structure to semicolon separated string and set the graphics style property
	mfprintf(fileID, "    // Convert to Scilab 6 suitable format\n");
	mfprintf(fileID, "    string_style = struct2string(style);\n");
    mfprintf(fileID, "    x_sci.graphics.style = string_style;\n");
	mfprintf(fileID, "    \n");
	
	// *** convert in- and out style to defined format
	mfprintf(fileID, "    // Convert to Scilab 6 suitable format\n");
	mfprintf(fileID, "    x_sci.graphics.in_style = getPortLabelStyle(define_in_style);\n");
    mfprintf(fileID, "    x_sci.graphics.out_style = getPortLabelStyle(define_out_style);\n");
    
    mfprintf(fileID, "end\n");
    mfprintf(fileID, "\n");
    mfprintf(fileID, "endfunction\n");
    
    // close file
    mclose(fileID);
    mprintf("Interface function %s successfully written.\n", fileName);

catch
    // some error creating the file occured
    mclose(fileID); // try to close the file
    mprintf("Error creating interface function for block %s: %s\n", blockInfo.blockName, lasterror());
end

endfunction
