// Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 1603 $
// $LastChangedDate:: 2019-01-21 19:02:13 +0100#$
// 
// This function merges 2 structures (the fields of each structure) and returns the resulting structure.
// If a structure field exists in both, the source and the adding structure, the field in the source structure
// is overwritten by the field from the additional structure.
// Example: merged_struct = mergeStructures(struct1, struct2)
// 
// Parameters:
// src_struct ... Source structure
// add_struct ... Additional structure being merged with the source structure
// 
// Return values:
// merged_struct ... Resulting merged structure of source- and additional structure

function [merged_struct] = mergeStructures(src_struct, add_struct)
    funcprot(0);

    merged_struct = src_struct;
    f_names_add = fieldnames(add_struct);
    // merge structures only if additional structure contains fields
    if ~isempty(f_names_add)
        execstr("merged_struct." + f_names_add + " = " + "add_struct." + f_names_add);
    end
endfunction
