// Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 3142 $
// $LastChangedDate:: 2024-03-15 16:03:31 +0100#$
//
// Starts Communicator in a dedicated Java VM and connects to it via Java RMI.

function [] = startCommunicatorRMI()
    jimport("at.lcm.x2c.utils.Utils");
    // Workaround using the 'host' command has to be used since Scilab 6.1.0 because of a Scilab bug:
    // Executing the Java Runner within the Scilab VM may crash Scilab in case of Scilab user interactions while the Communicator GUI is running */
    x2c_root = jinvoke(Utils, "getRootDirectory");
    // checks if X2C root variable does exist and if it points to a valid directory
    if type(x2c_root) ~= 17 then
        mprinf("Error: X2C root variable not set");
        return;
    elseif ~jinvoke(x2c_root, "isDirectory") then
        mprinf("Error: X2C root variable points to a non-existent location");
        return;
    end
    
    // Disable Communicator Model Auto Load feature to avoid possible Model Overwrite
    setenv('X2C_COMMUNICATOR_CONFIG_OPTIONS', 'DisableStartupModelLoad');
    
    java_bin = """" + jre_path() + filesep() + "bin" + filesep() + "java" + """";
    core_jar = """" + x2c_root + "System" + filesep() + "Java" + filesep() + "Core.jar" + """";
    class = "at.lcm.x2c.gui.communicator.RmiCommunicatorManager";
    method_name = "run";
    command = java_bin + " -cp " + core_jar + " " + "at.lcm.x2c.utils.JavaUtils" + " " + method_name + " " + class + " " + """" + jre_path() + """";
    
    // execute JRE in a new process that the environment variable 'X2C_COMMUNICATOR_CONFIG_OPTIONS' set by 'setenv' is inherited to the new process
    exit_code = host(command);
    // check OS exit code if Communicator GUI was successfully started 
    if (exit_code > 0) then
        mprintf("Error starting X2C Communicator\n");
    end
    
endfunction
